package com.example.playertime;

import com.mojang.brigadier.arguments.IntegerArgumentType;
import net.fabricmc.api.ModInitializer;
import net.fabricmc.fabric.api.command.v2.CommandRegistrationCallback;
import net.fabricmc.fabric.api.event.lifecycle.v1.ServerLifecycleEvents;
import net.fabricmc.fabric.api.networking.v1.ServerPlayConnectionEvents;
import net.fabricmc.loader.api.FabricLoader;
import net.fabricmc.loader.api.ModContainer;
import net.fabricmc.loader.api.Version;
import net.minecraft.server.command.CommandManager;
import net.minecraft.server.command.ServerCommandSource;
import net.minecraft.server.network.ServerPlayerEntity;
import net.minecraft.text.ClickEvent;
import net.minecraft.text.MutableText;
import net.minecraft.text.Text;
import net.minecraft.util.Util;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.DocumentBuilder;
import org.w3c.dom.Document;
import org.w3c.dom.NodeList;
import org.w3c.dom.Element;
import java.net.URL;
import java.util.List;
import java.util.Map;
import java.util.Optional;
import java.util.concurrent.CompletableFuture;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.stream.Collectors;

public class PlayerTimeMod implements ModInitializer {
    public static final Logger LOGGER = LoggerFactory.getLogger("PlayerTimeTracker");
    private static PlayerTimeTracker timeTracker;
    private static WebServer webServer;
    private static ModConfig config;
    public static LocalizationManager localizationManager;

    // RSS Feed URL for releases
    private static final String RSS_FEED_URL = "https://git.branulf.top/Branulf/ServerPlayerOnlineTracker/releases.rss";
    // Executor for update check to avoid blocking server startup
    private static final ExecutorService updateCheckExecutor = Executors.newSingleThreadExecutor();


    @Override
    public void onInitialize() {
        config = new ModConfig(FabricLoader.getInstance().getConfigDir());
        localizationManager = new LocalizationManager(config.getLanguage());

        try {
            LOGGER.info("[在线时间] 初始化 玩家在线时长视奸Mod");

            // Check for updates asynchronously on mod initialization
            checkForUpdates();

            // 在 SERVER_STARTING 阶段创建 Tracker 和 WebServer 实例
            ServerLifecycleEvents.SERVER_STARTING.register(server -> {
                timeTracker = new PlayerTimeTracker(server); // Tracker 构造函数不再加载数据
                try {
                    webServer = new WebServer(timeTracker, config.getWebPort(), server);
                    webServer.start();
                    LOGGER.info("[在线时间] Web服务器在端口 " + config.getWebPort() + "启动");
                } catch (Exception e) {
                    LOGGER.error("[在线时间] 无法启动Web服务器", e);
                }
            });

            // 在 SERVER_STARTED 阶段加载数据 (此时 UserCache 应该已可用)
            ServerLifecycleEvents.SERVER_STARTED.register(server -> {
                if (timeTracker != null) {
                    timeTracker.loadData(); // 在服务器启动完成后加载数据
                } else {
                    LOGGER.error("[在线时间] PlayerTimeTracker 未在 SERVER_STARTING 阶段成功初始化!");
                }
            });


            ServerPlayConnectionEvents.JOIN.register((handler, sender, server) -> {
                if (timeTracker != null) {
                    timeTracker.onPlayerJoin(handler.player);
                }
            });

            ServerPlayConnectionEvents.DISCONNECT.register((handler, server) -> {
                if (timeTracker != null) {
                    timeTracker.onPlayerLeave(handler.player);
                }
            });

            ServerLifecycleEvents.SERVER_STOPPING.register(server -> {
                LOGGER.info("[在线时间] 服务器停止 - 正在保存数据");

                if (webServer != null) {
                    webServer.stop();
                }
                if (timeTracker != null) {
                    // 在服务器停止前，确保所有在线玩家的会话时间被记录
                    for (ServerPlayerEntity player : server.getPlayerManager().getPlayerList()) {
                        timeTracker.onPlayerLeave(player); // onPlayerLeave 会自动保存该玩家数据
                    }
                    // timeTracker.saveAll(); // onPlayerLeave 已经异步保存，这里可以考虑是否还需要 saveAll
                    // 简单的处理是直接调用 saveAll，它会覆盖旧数据
                    timeTracker.saveAll();
                }
                // Shutdown the update check executor
                updateCheckExecutor.shutdownNow();
            });

        } catch (Exception e) {
            LOGGER.error("[在线时间] Mod初始化失败", e);
            throw new RuntimeException("[在线时间] Mod初始化失败 ", e);
        }
        registerCommands();
    }

    public static ModConfig getConfig() {
        return config;
    }

    public static PlayerTimeTracker getTimeTracker() {
        return timeTracker;
    }

    public static LocalizationManager getLocalizationManager() {
        return localizationManager;
    }

    public static void registerCommands() {
        CommandRegistrationCallback.EVENT.register((dispatcher, registryAccess, environment) -> {
            dispatcher.register(CommandManager.literal("onlineTime")
                    .requires(source -> source.hasPermissionLevel(0))
                    .executes(context -> showOnlineTime(context.getSource(), 1))
                    .then(CommandManager.argument("page", IntegerArgumentType.integer(1))
                            .executes(context -> showOnlineTime(
                                    context.getSource(),
                                    IntegerArgumentType.getInteger(context, "page")
                            ))
                    )
            );
        });
    }

    private static int showOnlineTime(ServerCommandSource source, int requestedPage) {
        // Commands are typically executed after SERVER_STARTED, so UserCache should be available here.
        ServerPlayerEntity player = source.getPlayer();
        if (player == null) return 0;

        // Use the server's main worker executor for command processing
        Util.getMainWorkerExecutor().execute(() -> {
            PlayerTimeTracker tracker = getTimeTracker();
            if (tracker != null) {
                Map<String, String> stats = tracker.getWhitelistedPlayerStats();

                List<String> sorted = stats.entrySet().stream()
                        .sorted((a, b) -> comparePlayTime(a.getValue(), b.getValue()))
                        .map(entry -> formatPlayerTime(entry.getKey(), entry.getValue()))
                        .collect(Collectors.toList());

                sendPaginatedMessage(player, sorted, requestedPage);
            } else {
                player.sendMessage(Text.literal(localizationManager.getString("playertime.command.error.not_initialized")), false);
            }
        });


        return 1;
    }

    private static int comparePlayTime(String a, String b) {
        // This parsing logic is specific to the format generated by getWhitelistedPlayerStats
        // Example: "PlayerName: 10h 30m | 7d: 5h 15m | 30d: 20h 45m"
        // We need to extract the total time part (e.g., "10h 30m")
        try {
            // Find the first colon and the first pipe
            int firstColon = a.indexOf(':');
            int firstPipe = a.indexOf('|');
            String timeA;
            if (firstColon > 0) {
                timeA = (firstPipe > firstColon && firstPipe != -1) ? a.substring(firstColon + 1, firstPipe).trim() : a.substring(firstColon + 1).trim();
            } else {
                timeA = a.trim(); // Fallback if format changes unexpectedly
            }


            int firstColonB = b.indexOf(':');
            int firstPipeB = b.indexOf('|');
            String timeB;
            if (firstColonB > 0) {
                timeB = (firstPipeB > firstColonB && firstPipeB != -1) ? b.substring(firstColonB + 1, firstPipeB).trim() : b.substring(firstColonB + 1).trim();
            } else {
                timeB = b.trim(); // Fallback if format changes unexpectedly
            }


            return Long.compare(parseTimeToSeconds(timeB), parseTimeToSeconds(timeA)); // Descending order
        } catch (Exception e) {
            LOGGER.error("Error comparing play times: {} vs {}", a, b, e);
            return 0; // Fallback to equal if parsing fails
        }
    }


    private static long parseTimeToSeconds(String timeStr) {
        long seconds = 0;
        // Handle potential empty or null strings
        if (timeStr == null || timeStr.trim().isEmpty()) {
            return 0;
        }

        String[] parts = timeStr.trim().split("\\s+"); // Split by one or more spaces
        for (String part : parts) {
            if (part.endsWith("h")) {
                try {
                    seconds += Integer.parseInt(part.substring(0, part.length() - 1)) * 3600;
                } catch (NumberFormatException ignored) {}
            } else if (part.endsWith("m")) {
                try {
                    seconds += Integer.parseInt(part.substring(0, part.length() - 1)) * 60;
                } catch (NumberFormatException ignored) {}
            }
            // Ignore other parts like "d:" or numbers without units
        }
        return seconds;
    }

    private static String formatPlayerTime(String name, String timeStr) {
        // timeStr is already formatted like "Total: Xh Ym | 7d: ... | 30d: ..."
        // We just need to prepend the player name
        return String.format("§e%s§r: %s", name, timeStr);
    }


    private static void sendPaginatedMessage(ServerPlayerEntity player, List<String> lines, int page) {
        int pageSize = 10;
        int totalPages = (lines.size() + pageSize - 1) / pageSize;
        page = Math.max(1, Math.min(page, totalPages));

        int from = (page - 1) * pageSize;
        int to = Math.min(from + pageSize, lines.size());

        player.sendMessage(Text.literal(localizationManager.getString("playertime.command.title", page, totalPages)), false);

        if (lines.isEmpty()) {
            player.sendMessage(Text.literal(localizationManager.getString("playertime.command.empty_stats")), false);
            return;
        }

        for (int i = from; i < to; i++) {
            player.sendMessage(Text.literal(lines.get(i)), false);
        }

        MutableText footer = Text.literal("");
        if (page > 1) {
            int finalPage = page;
            footer.append(Text.literal(localizationManager.getString("playertime.command.prev_page"))
                    .styled(style -> style.withClickEvent(new ClickEvent(
                            ClickEvent.Action.RUN_COMMAND,
                            "/onlineTime " + (finalPage - 1)
                    )))
                    .append(" "));
        }

        footer.append(Text.literal(localizationManager.getString("playertime.command.total_players", lines.size())));

        if (page < totalPages) {
            int finalPage1 = page;
            footer.append(" ").append(Text.literal(localizationManager.getString("playertime.command.next_page"))
                    .styled(style -> style.withClickEvent(new ClickEvent(
                            ClickEvent.Action.RUN_COMMAND,
                            "/onlineTime " + (finalPage1 + 1)
                    ))));
        }

        player.sendMessage(footer, false);
    }

    /**
     * Checks the RSS feed for new releases asynchronously.
     */
    private void checkForUpdates() {
        updateCheckExecutor.submit(() -> {
            try {
                // Get current mod version
                Optional<ModContainer> modContainer = FabricLoader.getInstance().getModContainer("playertime");
                if (modContainer.isEmpty()) {
                    LOGGER.warn("[在线时间] 无法获取 Mod 容器，跳过更新检查。");
                    return;
                }
                Version currentVersion = modContainer.get().getMetadata().getVersion();
                String currentVersionString = currentVersion.getFriendlyString();

                LOGGER.info("[在线时间] 正在检查更新...");

                // Fetch and parse RSS feed
                DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
                DocumentBuilder builder = factory.newDocumentBuilder();
                Document doc = builder.parse(new URL(RSS_FEED_URL).openStream());
                doc.getDocumentElement().normalize();

                NodeList itemList = doc.getElementsByTagName("item");

                if (itemList.getLength() == 0) {
                    LOGGER.warn("[在线时间] RSS Feed 中没有找到任何发布项。");
                    return;
                }

                // Get the latest item (first one in the feed)
                Element latestItem = (Element) itemList.item(0);
                String latestVersionString = latestItem.getElementsByTagName("title").item(0).getTextContent();
                String latestVersionLink = latestItem.getElementsByTagName("link").item(0).getTextContent();

                LOGGER.info("[在线时间] 当前版本: {}, 最新版本 (RSS): {}", currentVersionString, latestVersionString);

                // Compare versions
                if (isNewerVersion(latestVersionString, currentVersionString)) {
                    LOGGER.warn("==================================================");
                    LOGGER.warn("[在线时间] 发现新版本!");
                    LOGGER.warn("[在线时间] 当前版本: {}", currentVersionString);
                    LOGGER.warn("[在线时间] 最新版本: {}", latestVersionString);
                    LOGGER.warn("[在线时间] 下载链接: {}", latestVersionLink);
                    LOGGER.warn("==================================================");
                } else {
                    LOGGER.info("[在线时间] 当前已是最新版本(也有可能检查失败)。");
                }

            } catch (javax.xml.parsers.ParserConfigurationException e) {
                LOGGER.error("[在线时间] 更新检查失败: XML 解析器配置错误", e);
            } catch (org.xml.sax.SAXException e) {
                LOGGER.error("[在线时间] 更新检查失败: XML 解析错误", e);
            } catch (java.io.IOException e) {
                LOGGER.error("[在线时间] 更新检查失败: 网络或文件读取错误", e);
            } catch (Exception e) {
                LOGGER.error("[在线时间] 更新检查时发生未知错误", e);
            }
        });
    }

    /**
     * Compares two version strings (e.g., "1.2.3" vs "1.2.4").
     * Returns true if newVersion is newer than currentVersion.
     * Assumes versions are dot-separated integers.
     */
    private boolean isNewerVersion(String newVersion, String currentVersion) {
        if (newVersion == null || currentVersion == null || newVersion.isEmpty() || currentVersion.isEmpty()) {
            return false; // Cannot compare
        }

        // Clean up version strings - remove potential prefixes like "v"
        String cleanNewVersion = newVersion.toLowerCase().startsWith("v") ? newVersion.substring(1) : newVersion;
        String cleanCurrentVersion = currentVersion.toLowerCase().startsWith("v") ? currentVersion.substring(1) : currentVersion;


        String[] newParts = cleanNewVersion.split("\\.");
        String[] currentParts = cleanCurrentVersion.split("\\.");

        int maxLength = Math.max(newParts.length, currentParts.length);

        for (int i = 0; i < maxLength; i++) {
            int newPart = (i < newParts.length) ? parseIntOrZero(newParts[i]) : 0;
            int currentPart = (i < currentParts.length) ? parseIntOrZero(currentParts[i]) : 0;

            if (newPart > currentPart) {
                return true; // New version is newer
            }
            if (newPart < currentPart) {
                return false; // New version is older
            }
            // If parts are equal, continue to the next part
        }

        return false; // Versions are equal
    }

    /**
     * Parses a string part of a version into an integer, returning 0 if parsing fails.
     */
    private int parseIntOrZero(String s) {
        try {
            return Integer.parseInt(s);
        } catch (NumberFormatException e) {
            return 0; // Treat non-numeric parts as 0 for comparison
        }
    }
}
